@@ -10,6 +10,9 @@ from pathlib import Path
 from traceback import format_exception
 from warnings import warn
 import signal
+import os
+import subprocess
+import platform
 
 import docstring_parser as dp
 import pandas as pd
@@ -24,6 +27,7 @@ from .typeoverloads import FilePath
 
 # -----
 # For Pyyaml
+# -----
 # import yaml
 # from yaml import YAMLError
 # def yamlLoad(stream, loader=yaml.UnsafeLoader):
@@ -35,7 +39,7 @@ from .typeoverloads import FilePath
 # For ruamel yaml
 # -----
 from ruamel.yaml import YAML, YAMLError
-yaml = YAML(typ='unsafe')
+yaml = YAML()
 def yamlLoad(stream):
   return yaml.load(stream)
 def yamlDump(data, stream):
@@ -640,4 +644,13 @@ def hierarchicalUpdate(curDict: dict, other: dict, replaceLists=False):
       curVal.extend(v)
     else:
       curDict[k] = v
-  return curDict
\ No newline at end of file
+  return curDict
+
+def openInEditor(filepath: FilePath):
+  filepath = str(filepath)
+  if platform.system() == 'Darwin':       # macOS
+      subprocess.call(('open', filepath))
+  elif platform.system() == 'Windows':    # Windows
+      os.startfile(filepath)
+  else:                                   # linux variants
+      subprocess.call(('xdg-open', filepath))
\ No newline at end of file
