@@ -134,13 +134,7 @@ class ParamEditor(QtWidgets.QDockWidget):
       self.saveDir.mkdir(parents=True, exist_ok=True)
       self.watcher.addPath(str(self.saveDir))
     self.watcher.directoryChanged.connect(lambda: self.updateSavedStates())
-    def maybeUpdateCurState(file: str):
-      if Path(file) == self.formatFileName(self.stateName):
-        # Time for a reload, Put in event loop to allow file time to save
-        # TODO: 5 ms might not be long enough for a file save, figure out a good
-        #   way of waiting the right amount of time
-        QtCore.QTimer.singleShot(5, lambda: self.loadParamValues(self.stateName))
-    self.watcher.fileChanged.connect(maybeUpdateCurState)
+    self.watcher.fileChanged.connect(self._maybeUpdateCurState)
 
     # -----------
     # Construct parameter tree
@@ -186,6 +180,14 @@ class ParamEditor(QtWidgets.QDockWidget):
 
     SPAWNED_EDITORS.append(weakref.proxy(self))
 
+  def _maybeUpdateCurState(self, file: str):
+    """Defined as an outer function since it has to be disabled sometimes (e.g. during save to same name)"""
+    if Path(file) == self.formatFileName(self.stateName):
+      # Time for a reload, Put in event loop to allow file time to save
+      # TODO: 5 ms might not be long enough for a file save, figure out a good
+      #   way of waiting the right amount of time
+      QtCore.QTimer.singleShot(5, lambda: self.loadParamValues(self.stateName))
+
   def applyChanges(self, newName: FilePath=None, newState: dict=None, recreate=False):
     """
     Broadcasts that this parameter editor has updated changes
@@ -206,11 +208,6 @@ class ParamEditor(QtWidgets.QDockWidget):
 
     if self.stateName == newName:
       return
-    oldName = self.formatFileName(self.stateName)
-    # Watch the new name, take out the old name. OK if neither exists, will just return
-    # false
-    self.watcher.removePath(str(oldName))
-    self.watcher.addPath(str(self.formatFileName(newName)))
 
     self.stateName = newName
     self.stateSelector.setValue(self.stateName)
@@ -231,8 +228,13 @@ class ParamEditor(QtWidgets.QDockWidget):
       applyChanges = False
     # Remove non-useful values
     if not blockWrite and self.saveDir is not None:
-      fns.saveToFile(paramState, self.formatFileName(saveName),
-                 allowOverwriteDefault=allowOverwriteDefault)
+      # This shouldn't cause a self-reload
+      self.watcher.fileChanged.disconnect(self._maybeUpdateCurState)
+      try:
+        fns.saveToFile(paramState, self.formatFileName(saveName),
+                   allowOverwriteDefault=allowOverwriteDefault)
+      finally:
+        self.watcher.fileChanged.connect(self._maybeUpdateCurState)
     if applyChanges:
       # Since save state shouldn't be modifying the current state, it will emit the same
       # state as before, but under a new name -- regardless of the param state dict
