@@ -36,7 +36,7 @@ class ProcessIO(dict):
     latter IO will not make use of `FROM_PREV_IO`.
     """
 
-  def __init__(self, hyperParamKeys: t.Sequence[str]=None, **kwargs) -> None:
+  def __init__(self, hyperParamKeys: t.List[str]=None, **kwargs) -> None:
     """
     :param hyperParamKeys: Hyperparameters for this process that aren't expected to come
       from the previous stage in the process. Forwarded keys (that were passed from a
@@ -57,6 +57,7 @@ class ProcessIO(dict):
            f'Defaulting to `None` for those keys.', UserWarning)
     super().__init__(**kwargs)
     self.keysFromPrevIO = set(self.keys()) - set(self.hyperParamKeys)
+    self.hasKwargs = False
 
   @classmethod
   def fromFunction(cls, func: t.Callable, ignoreKeys: StrCol=None, **overriddenDefaults):
@@ -77,9 +78,13 @@ class ProcessIO(dict):
     outDict = {}
     hyperParamKeys = []
     spec = inspect.signature(func).parameters
+    checkUnclaimedOverrides = False
     for k, v in spec.items():
       if (k in ignoreKeys
-              or v.kind in (v.VAR_POSITIONAL, v.VAR_KEYWORD)):
+              or v.kind is v.VAR_POSITIONAL):
+        continue
+      elif v.kind is v.VAR_KEYWORD:
+        checkUnclaimedOverrides = True
         continue
       formattedV = overriddenDefaults.get(k, v.default)
       if formattedV is v.empty:
@@ -88,6 +93,13 @@ class ProcessIO(dict):
       else:
         hyperParamKeys.append(k)
       outDict[k] = formattedV
+    # Functions accepting kwargs can be given unnamed arguments from overridden defaults
+    if checkUnclaimedOverrides:
+      unclaimedOverrideKeys = set(overriddenDefaults) - set(hyperParamKeys)
+      for k in unclaimedOverrideKeys:
+        outDict[k] = overriddenDefaults[k]
+        # TODO: How to determine whether these should be hyperparams as well? For now, since they cannot have
+        #   docs to accompany them, consider them non-hyperparams
     # Make sure to avoid 'init got multiple values' error
     initSig = inspect.signature(cls.__init__).parameters
     keys = list(outDict)
@@ -96,7 +108,10 @@ class ProcessIO(dict):
         newName = '_' + key
         outDict[newName] = outDict[key]
         del outDict[key]
-    return cls(hyperParamKeys, **outDict)
+    out = cls(hyperParamKeys, **outDict)
+    if checkUnclaimedOverrides:
+      out.hasKwargs = True
+    return out
 
 class ProcessStage(ABC):
   name: str
@@ -128,29 +143,17 @@ class ProcessStage(ABC):
   def __hash__(self):
     return id(self)
 
-  def updateInput(self, prevIo: ProcessIO=None, graceful=False, **kwargs):
+  def updateInput(self, prevIo: ProcessIO=None, graceful=False, allowExtra=False, **kwargs):
     """
     Helper function to update current inputs from previous ones while ignoring leading
     underscores.
 
     :param prevIo: Io object to forward updated inputs here. Extra inputs can be supplied to **kwargs
     :param graceful: If *True*, doesn't error on missing keys or bad update hierarchy
+    :param allowExtra: Whether to allow setting of keys that didn't exist in the original input. This can be valid
+      in the case where the underlying function accepts **kwargs
     """
-    if prevIo is None:
-      prevIo = ProcessIO()
-    useIo = prevIo.copy()
-    useIo.update(kwargs)
-    selfFmtToUnfmt = {k.lstrip('_'): k for k in self.input}
-    requiredKeyFmt = {k: v for k, v in selfFmtToUnfmt.items() if v in self.input.keysFromPrevIO}
-    prevIoKeyToFmt = {k.lstrip('_'): k for k in {**useIo}}
-    missingKeys = []
-    for fmtK, trueK in selfFmtToUnfmt.items():
-      if fmtK in prevIoKeyToFmt:
-        self.input[trueK] = useIo[prevIoKeyToFmt[fmtK]]
-      elif fmtK in requiredKeyFmt:
-        missingKeys.append(fmtK)
-    if missingKeys and not graceful:
-      raise KeyError(f'Missing Following keys from {self}: {missingKeys}')
+    raise NotImplementedError
 
   def run(self, io: ProcessIO=None, disable=False, **runKwargs):
     raise NotImplementedError
@@ -216,6 +219,7 @@ class AtomicProcess(ProcessStage):
 
   def __init__(self, func: t.Callable, name:str=None, *, needsWrap=False,
                mainResultKeys: StrList=None, mainInputKeys: StrList=None,
+               docFunc: t.Callable=None,
                **procIoKwargs):
     """
     :param func: Function to wrap
@@ -231,18 +235,30 @@ class AtomicProcess(ProcessStage):
     the output is expected to be the direct result, not a sequence of results per key.
     :param mainResultKeys: Set by parent process as needed
     :param mainInputKeys: Set by parent process as needed
+    :param docFunc: Sometimes, `func` is a wrapper around a different function, where this other function is what should
+      determine the created user parameters. In these cases (i.e. one function directly calling another), that inner
+      function can be provided here for parsing the docstring and parameters. This is only used for the purpose of
+      creationg a function specification and is ignored afterward.
     :param procIoKwargs: Passed to ProcessIO.fromFunction
     """
     if name is None:
-      name = nameFormatter(func.__name__)
+      name = nameFormatter((docFunc or func).__name__)
     if mainResultKeys is not None:
       self.mainResultKeys = mainResultKeys
     if mainInputKeys is not None:
       self.mainInputKeys = mainInputKeys
 
     self.name = name
-    self.input = ProcessIO.fromFunction(func, **procIoKwargs)
+    try:
+      self.input = ProcessIO.fromFunction(docFunc or func, **procIoKwargs)
+    except ValueError:
+      # Happens on builtins / c-defined functions. Assume user passes all meaningful args at startup
+      self.input = ProcessIO(**procIoKwargs)
+      self.input.hyperParamKeys = {k for k, v in self.input.items() if v is not None}
+    self.hasKwargs = self.input.hasKwargs
     self.result: t.Optional[ProcessIO] = None
+    self.fnDoc = (docFunc or func).__doc__
+    """Record function documentation of proxy if needed so wrapper functions properly"""
 
     if mainInputKeys is not None:
       keys = set(self.input.keys())
@@ -280,7 +296,6 @@ class AtomicProcess(ProcessStage):
     return self.input.keysFromPrevIO
 
   def run(self, io: ProcessIO=None, disable=False, **runKwargs):
-    ProcessIO.count = self
     disable = disable or self.disabled
     self.updateInput(io, **runKwargs)
     if not disable:
@@ -292,6 +307,31 @@ class AtomicProcess(ProcessStage):
       out = self.input
     return out
 
+  def updateInput(self, prevIo: ProcessIO=None, graceful=False, allowExtra=False, **kwargs):
+    allowExtra = allowExtra or self.hasKwargs
+    if prevIo is None:
+      prevIo = ProcessIO()
+    useIo = prevIo.copy()
+    useIo.update(kwargs)
+    selfFmtToUnfmt = {k.lstrip('_'): k for k in self.input}
+    requiredKeyFmt = {k for k, v in selfFmtToUnfmt.items() if v in self.input.keysFromPrevIO}
+    prevIoKeyToFmt = {k.lstrip('_'): k for k in {**useIo}}
+    setKeys = set()
+    for fmtK, value in zip(prevIoKeyToFmt, useIo.values()):
+      if fmtK in selfFmtToUnfmt:
+        trueK = selfFmtToUnfmt[fmtK]
+        self.input[trueK] = value
+        setKeys.add(fmtK)
+      elif allowExtra:
+        # No true key already exists, so assume formatting leads to desired name
+        self.input[fmtK] = value
+        # Not possible for this to be required, so no need to add to set keys
+
+    missingKeys = (requiredKeyFmt - setKeys)
+    if missingKeys and not graceful:
+      # Some required keys were not given
+      raise KeyError(f'Missing Following keys from {self}: {missingKeys}')
+
   def saveState(self, includeDefaults=False, includeMeta=False, **metaFilterOpts):
     def keepCond(k, v):
       return (k in self.input.hyperParamKeys or k not in self.defaultInput) \
@@ -355,7 +395,7 @@ class NestedProcess(ProcessStage):
     self.stages.append(process)
     return process
 
-  def updateInput(self, prevIo: ProcessIO=None, graceful=False, **kwargs):
+  def updateInput(self, prevIo: ProcessIO=None, graceful=False, allowExtra=False, **kwargs):
     """Drill down to the necessary stages when setting input values when setter values are dicts"""
     if prevIo is None:
       prevIo = ProcessIO()
@@ -369,10 +409,10 @@ class NestedProcess(ProcessStage):
           raise ValueError(f'Name {kk} doesn\'t match existing stages')
         elif not matchStage:
           continue
-        matchStage[0].updateInput(**vv, graceful=graceful)
+        matchStage[0].updateInput(**vv, graceful=graceful, allowExtra=allowExtra)
         prevIo.pop(kk)
     # Normal input maps to first stage, so propagate all normal args there
-    self.stages[0].updateInput(prevIo, graceful=graceful)
+    self.stages[0].updateInput(prevIo, graceful=graceful, allowExtra=allowExtra)
 
   def run(self, io: ProcessIO = None, disable=False, **runKwargs):
     _activeIo = ProcessIO() if not io else _copy.copy(io)
@@ -383,7 +423,14 @@ class NestedProcess(ProcessStage):
       return _activeIo
 
     for i, stage in enumerate(self):
-      newIo = stage.run(_activeIo, disable=disable or self.disabled)
+      try:
+        newIo = stage.run(_activeIo, disable=disable or self.disabled)
+      except Exception as ex:
+        # Provide information about which stage failed
+        if not isinstance(ex.args, tuple):
+          ex.args = (ex.args,)
+        ex.args = (stage,) + ex.args
+        raise
       if not isinstance(newIo, ProcessIO) and isinstance(stage, AtomicProcess):
         # Try wrapping the inner function for future runs
         newIo = self._maybeWrapStageFunc(stage, newIo)
@@ -422,7 +469,7 @@ class NestedProcess(ProcessStage):
 
   @property
   def input(self):
-    return self.stages[0].input
+    return self.stages[0].input if self.stages else None
 
   def flatten(self, copy=True, includeDisabled=True):
     if copy:
@@ -523,4 +570,31 @@ class NestedProcess(ProcessStage):
           if np.array_equal(info[key], lastInfo[key]):
             validInfo[key] = cls._DUPLICATE_INFO
       validInfos.append(validInfo)
-    return validInfos
\ No newline at end of file
+    return validInfos
+
+class ArgMapper(AtomicProcess):
+  """
+  Used to convert the named outputs of a previous stage into different names for process compatibility. Meant to be used
+  within a NestedProcess. Specify kwargs in the form of {outKeyName: inKeyName}.
+  """
+
+  def __init__(self, **kwargs):
+    # Just give a dummy function accepting overridable kwargs
+    kwargs['func'] = self.argMapper
+    super().__init__(**kwargs)
+    self.forwardOthers = True
+
+  def updateInput(self, prevIo: ProcessIO = None, graceful=False, allowExtra=True, **kwargs):
+    return super().updateInput(prevIo, graceful, allowExtra, **kwargs)
+
+  def argMapper(self, **kwargs):
+    if self.forwardOthers:
+      out = ProcessIO(**kwargs)
+      for kk in self.defaultInput.values():
+        # Remove keys that are mapped to new names
+        del out[kk]
+    else:
+      out = ProcessIO()
+    for kk, vv in self.defaultInput.items():
+      out[kk] = kwargs[vv]
+    return out
\ No newline at end of file
